/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
/*
 *  (C) 2001 by Argonne National Laboratory.
 *      See COPYRIGHT in top-level directory.
 */

/* Test program for managing locmemtabsize */

#include "mpi.h"
#include "dmem.h"

char* a2s(long);
char* n2s(long);

static MPI_Datatype handle_type;

#define NUM_IRECVS 5

int main(int argc, char *argv[])
{
    int i, j, mpi_rank, mpi_size, provided;
    MPI_Aint memsize;
    char pkt0[1000], pkt1[2000], pkt2[3000], pkt4[1000], *buf;
    dmem_handle dh0, dh1, dh2;
    MPI_Comm dmem_comm;

    MPI_Init_thread(0, 0, MPI_THREAD_MULTIPLE, &provided);
    if (provided != MPI_THREAD_MULTIPLE) {
        printf("insufficient level of thread safety provided by MPI implementation\n");
        MPI_Abort(MPI_COMM_WORLD,777);
    }
    MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
    if (mpi_size != 2) {
        printf("userloctabtest requires exactly 2 processes\n");
        MPI_Finalize();
        return(0);
    }
    MPI_Comm_dup(MPI_COMM_WORLD, &dmem_comm);

    MPI_Type_vector(3, 1, 1, MPI_AINT, &handle_type);
    MPI_Type_commit(&handle_type);

    if (mpi_rank == 0) {	/* DMEM manager */
        DMEM_Init(dmem_comm, 100, NUM_IRECVS);
    }
    else {
        DMEM_Init(dmem_comm, 30000, NUM_IRECVS);  /* plenty of room */
        for (i = 0; i < 1000; i++) pkt0[i] = '0'; /* initialize block */

        for (j = 0; j < 12; j++) { /* try to overflow locmemtab, not locmem */
            printf("rank %d putting packet %d\n", mpi_rank, j);
            DMEM_Put(pkt0, 10, dh0);
        }
    }
    DMEM_Finalize();
    MPI_Finalize();
    return(0);
}
      
