/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
/*
 *  (C) 2001 by Argonne National Laboratory.
 *      See COPYRIGHT in top-level directory.
 */

/* Exercizing MPI-3 shared memory routines */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "mpi.h"

int main(int argc, char *argv[])
{
    int size, rank, shmemsize, shmemrank;
    int key = 0;
    MPI_Aint memsize;
    MPI_Comm shmemcomm;
    void *baseptr;
    MPI_Win shwin;

    MPI_Init(0, 0);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);
    printf("rank %d of %d in MPI_COMM_WORLD\n", rank, size);

    MPI_Comm_split_type(MPI_COMM_WORLD, MPI_COMM_TYPE_SHARED, key, MPI_INFO_NULL, &shmemcomm);

    MPI_Comm_rank(shmemcomm, &shmemrank);
    MPI_Comm_size(shmemcomm, &shmemsize);
    printf("rank %d of %d in shmemcomm is rank %d in MPI_COMM_WORLD\n", shmemrank, shmemsize, rank);

    /* Allocate the shared memory window*/
    memsize = 1024*1024;
    MPI_Win_allocate_shared(memsize, MPI_BYTE, MPI_INFO_NULL, shmemcomm, &baseptr, &shwin);
    printf("rank %d in shmemcomm got baseptr %p\n", shmemrank, baseptr);

    /* Test whether the memory is really shared */
    if (shmemrank == 0) {
        *(char *) baseptr = (char) 'a';
        printf("rank 0 in shmemcomm putting 'a' in shared memory\n");
        MPI_Barrier(shmemcomm);
    }
    else {
        MPI_Barrier(shmemcomm);
        printf("rank %d in shmemcomm reading %c from shared memory\n", shmemrank, *(char *) baseptr);
    }
    MPI_Finalize();

    return 0;
}  
  
